require 'logger'

# Dropper::Logger handles all the logging for the application. By default it will
# just log to STDOUT, but if you set an outpath
# (Dropper::Logger.outpath = "filename") then this class will log to both STDOUT
# and the given file.
class Dropper::Logger
  class << self
    attr_accessor :outpath

    def loggers
      @loggers ||= build_loggers
    end

    # Log Info
    # @param msg [String] The message to be logged
    def info(msg)
      log(:info, msg)
    end

    # Log Debug
    # @param msg [String] The message to be logged
    def debug(msg)
      log(:debug, msg)
    end

    # Log Fatal
    # @param msg [String] The message to be logged
    def fatal(msg)
      log(:fatal, msg)
    end

    # Log Warn
    # @param msg [String] The message to be logged
    def warn(msg)
      log(:warn, msg)
    end

    # Log Error
    # @param msg [String] The message to be logged
    def error(msg)
      log(:error, msg)
    end

    private

    def log(type, msg)
      loggers.each do |l|
        l.send(type, msg)
      end
    end

    def build_loggers
      logs = [Logger.new(STDOUT)]
      logs += [Logger.new(outpath)] if outpath


      logs[0].formatter = proc do |severity, datetime, progname, msg|
        span_message(msg, severity, nil)
      end

      if logs.size > 1
        logs[1].formatter = proc do |severity, datetime, progname, msg|

        end
      end

      logs
    end

    def span_message(msg, tag, tag_color)
      spaces = term_width - msg.length - tag.length - 4

      output = msg
      output += " " * spaces
      output += "[ #{tag} ]"

      output
    end

    def term_width
      detect_terminal_size[0].to_i
    end

    def detect_terminal_size
      if (ENV['COLUMNS'] =~ /^\d+$/) && (ENV['LINES'] =~ /^\d+$/)
        [ENV['COLUMNS'].to_i, ENV['LINES'].to_i]
      elsif (RUBY_PLATFORM =~ /java/ || (!STDIN.tty? && ENV['TERM'])) && command_exists?('tput')
        [`tput cols`.to_i, `tput lines`.to_i]
      elsif STDIN.tty? && command_exists?('stty')
        `stty size`.scan(/\d+/).map { |s| s.to_i }.reverse
      else
        nil
      end
    rescue
      nil
    end
  end
end
